// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
// Funded by CEA, DAM, DIF, F-91297 Arpajon, France
#ifndef vtkVTK_DISPATCH_IMPLICIT_ARRAYS_h
#define vtkVTK_DISPATCH_IMPLICIT_ARRAYS_h

// defined if VTK dispatches the vtkAffineArray class
/* #undef VTK_DISPATCH_AFFINE_ARRAYS */
// defined if VTK dispatches the vtkConstantArray class
/* #undef VTK_DISPATCH_CONSTANT_ARRAYS */
// defined if VTK dispatches the vtkStdFunctionArray class
/* #undef VTK_DISPATCH_STD_FUNCTION_ARRAYS */
// defined if VTK dispatches the vtkStridedArray class
/* #undef VTK_DISPATCH_STRIDED_ARRAYS */
// defined if VTK dispatches the vtkStructuredPointArray class
/* #undef VTK_DISPATCH_STRUCTURED_POINT_ARRAYS */

#endif // vtkVTK_DISPATCH_IMPLICIT_ARRAYS_h
